/**
* ======================================================================
*
*	File: mst_cards.js
*	Version: 1.0
*	Author: Wellcom Worldwide
*
*	Description: 
*
*	This file contains the cards functionality used for the 
*	Mercer Digital Style Guide.
*	
* ======================================================================
*/

;(function($) {



	// # Check: that jQuery exists.
	if(typeof $ == "undefined" && typeof jQuery == "undefined") {
		throw new Error('JQuery must be loaded.');
	} else if ( typeof $ == "undefined" && typeof jQuery != "undefined") {
		var $ = jQuery;
	}

	

	$.fn.__mstCards = function( opts ) {

		// # Initialize: the different selectors that are used
		var selectedObjects = this;

		// # Define: Default Plugin Settings
		var settings = $.extend({
			equalizeTitles: true,
			equalizeTitlesByRow: true,
			equalizeContent: true,
			enableFocusableTitles: true,
			enableKeypressCards: true,
			enableAccessibleVoice: true,
			mobileBreakpoint: 768
			// Silence is golden.
		}, opts);


		// # Define: Plugin Methods
		var pluginMethods = {
			// # Function: Initialization of Plugin
			__init: function( selectedObjects ) {
				
				// # Loop: Check each selected item has correct classes
				$.each(selectedObjects, function(index, object) {

					var selector = $(this);

					if( !selector.hasClass('__mst_cards')) {
						selector.addClass('__mst_cards');
					}


					// Now that the selector has the correct class applied, 
					// now perform the conversation.
					if( settings.equalizeTitles ) {

						var cardColumns = selector.find('.__mst_cards_column');

						if( settings.equalizeTitlesByRow ) {
							pluginMethods.equalizeTitlesByRow( selector );
						} else {
							pluginMethods.equalizeTitles( selector );
						}
					}

					if( settings.equalizeContent && $(window).width() > settings.mobileBreakpoint ) {
						pluginMethods.equalizeContent( selector );
					}

					// # Show: All Cards that have card-open class
					var cardColumn = selector.find('.__mst_cards_column');
					$.each(cardColumn, function( cardIndex, cardObject ) {
						var column = $(this);
						var cards = column.find('.__mst_card');
						for( var card = 0, totalCards = cards.length; card < totalCards; card++ ) {
							var currentCard = $( cards[ card ] );
							if( $( window ).width() > settings.mobileBreakpoint ) {
								if( currentCard.hasClass('card-open') && card == 0 ) {
									var cardContent = currentCard.find('.__mst_card_content');
									cardContent.show();
								} else {
									currentCard.removeClass('card-open');
								}
							} else {
								if( currentCard.hasClass('card-open') && card == 0 && cardIndex == 0 ) {
									var cardContent = currentCard.find('.__mst_card_content');
									cardContent.show();
								} else {
									currentCard.removeClass('card-open');
								}
							}
						}
					});

					// # Attach: On-Click Methods
					var cardTitles = selector.find( '.__mst_card_title_wrapper' );
					$.each( cardTitles, function( titleIndex, titleObject ) {
						var titleWrapper = $( this );
						titleWrapper.off('click tap', pluginMethods.onClickTitleEvent)
									.on('click tap', pluginMethods.onClickTitleEvent);
					});

					if( settings.enableFocusableTitles ) {
						pluginMethods.focusableTitles( selector );
					}

					if( settings.enableKeypressCards ) {

						var cards = $( selector ).find('.__mst_card');
						$.each( cards, function( cardIndex, cardObject ) {
							var currentCard = $( this );
							var cardTitle = currentCard.find('.__mst_card_title_wrapper');
							cardTitle.off('keypress', pluginMethods.bindKeypressEvent)
										.on('keypress', pluginMethods.bindKeypressEvent);
						});
					}

					if( settings.enableAccessibleVoice ) {
						selector.attr('role', 'region');
						// selector.attr('aria-multiselectable', 'true');

						// # Enable: Voice on all Card Elements
						var cards = $( selector ).find('.__mst_card');

						$.each( cards, function( cardIndex, cardObject ) {
							var currentCard = $( this );
							var iconSpanElement = currentCard.find('.__mst_card_caret');
							var titleTextElement = currentCard.find('.__mst_card_title_text');
							var titleTextWrapper = currentCard.find('.__mst_card_title_wrapper');
							var cardContent = currentCard.find('.__mst_card_content');
							var titleText = titleTextElement.text();
							var lastTitleTextChar = titleText.substr( titleText.length - 1 );
							var allowedTitleEndings = ['?', '!', '.'];

							titleText = allowedTitleEndings.indexOf( lastTitleTextChar ) === -1 ? titleText + "." : titleText;

							titleTextWrapper.attr('aria-label', titleText);
							titleTextWrapper.attr('title', titleText);
							titleTextWrapper.attr('role', 'button');

							if( currentCard.hasClass('card-open') ) {
								titleTextWrapper.attr('aria-expanded', 'true');
							} else {
								
								titleTextWrapper.attr('aria-expanded', 'false');
							}
						});


						// # Enable: Voice on content elements such as text/images
						var contentCotainers = selector.find('.__mst_cards_column_content');
						
						$.each( contentCotainers, function( contentIndex, contentObject ) {
							var currentContentContainer = $( this );
							currentContentContainer.attr('tabindex', '0');
						});

					}

				});

			},
			__reset: function( selectedObjects ) {
				$.each(selectedObjects, function(index, object) {
					var selector = $(this);
					var cardContent = selector.find('.__mst_card_content_inner');
					var titles = selector.find('.__mst_card_title');
					var titleText = selector.find('.__mst_card_title_text');
					cardContent.css('min-height', '');
					titles.removeClass('__mst_multiline_title');
					titleText.css('line-height', '');
					titleText.css('height', '');
				});
			},
			bindKeypressEvent: function ( keypressEv ) {

				// # Detect: Spacebar/Enter Keys to trigger event
				if( keypressEv.keyCode === 13 || keypressEv.keyCode === 32 ) {
					keypressEv.preventDefault();
					var currentCard = $( this );
					var target = $( keypressEv.target );

					if( target.hasClass('__mst_card_title_wrapper') ) {
						var card = target.parents('.__mst_card');
						var titleTextWrapper = card.find('.__mst_card_title_wrapper');
						card.toggleClass('card-open');
						var cardContent = card.find('.__mst_card_content');
						if( card.hasClass('card-open') ) {
							cardContent.slideDown();
							if( settings.enableAccessibleVoice ) {
								titleTextWrapper.attr('aria-expanded', 'true');
							}
						} else {
							cardContent.slideUp();
							if( settings.enableAccessibleVoice ) {
								titleTextWrapper.attr('aria-expanded', 'false');
							}
						}
					} else if ( target.hasClass('__mst_card_cta') ) {
						target.trigger('click');
					}
				}

			},
			onClickTitleEvent: function ( clickEv ) {

				var currentTitle = $(this);
				var card = currentTitle.parents('.__mst_card');

				card.toggleClass('card-open');
				var cardContent = card.find('.__mst_card_content');
				var iconSpanElement = card.find('.__mst_card_caret');
				var titleTextWrapper = card.find('.__mst_card_title_wrapper');
				
				if( card.hasClass('card-open') ) {
					cardContent.slideDown();
					if( settings.enableAccessibleVoice ) {
						titleTextWrapper.attr('aria-expanded', 'true');
					}
				} else {
					cardContent.slideUp();
					if( settings.enableAccessibleVoice ) {
						titleTextWrapper.attr('aria-expanded', 'false');
					}
				}

			},
			equalizeTitles: function( selector ) {

				var cardTitles = $( selector ).find('.__mst_card_title_text');

				// # Reset: All titles back to default
				cardTitles.css('line-height', '');

				var minHeight = 0;
				var useMinHeight = false;
				for( var title = 0, totalTitles = cardTitles.length; title < totalTitles; title++ ) {

					var currentTitleText = $( cardTitles[ title ] );
					var titleElemHeight = currentTitleText.height();
					var titleElemLineHeight = parseFloat( $( cardTitles[ title ] ).css('line-height') );
					var totalTitleLines = titleElemHeight / titleElemLineHeight;


					if( useMinHeight === false && totalTitleLines > 2 ) {
						console.log( 'Hit on : ' + currentTitleText.text() );
						useMinHeight = true;
					}

					if( $( cardTitles[ title ] ).height()  > minHeight ) {
						minHeight = $( cardTitles[ title ] ).height();
					}
				}

				for( var title = 0, totalTitles = cardTitles.length; title < totalTitles; title++ ) {
					var currentTitleText = $( cardTitles[ title ] );
					var currentCardTitleHeight = currentTitleText.height();
					var titleElem = currentTitleText.parents('.__mst_card_title');

					if( currentCardTitleHeight !== minHeight ) {
						if( useMinHeight === false ) {
							currentTitleText.css('line-height', minHeight + 'px');
							titleElem.removeClass('__mst_multiline_title');
						} else {
							currentTitleText.css('height', minHeight + 'px');
							titleElem.addClass('__mst_multiline_title');
						}
					}

				}

			},
			equalizeTitlesByRow: function( selector ) {

				var cardColumns = $( selector ).find('.__mst_cards_column');
				var nestedCards = [];
				var maxCardsPerColumn = 0;
				
				// # Determine: Amount of cards per column to determine rows.
				for( var column = 0, totalColumns = cardColumns.length; column < totalColumns; column++ ) {
					var totalCards = $( cardColumns[ column ] ).find('.__mst_card');
					if( totalCards.length > maxCardsPerColumn ) {
						maxCardsPerColumn = totalCards.length;
					}
				}

				// # Add: Empty arrays to make nested array based on each row
				for( var card = 0, totalCards = maxCardsPerColumn; card < totalCards; card++ ) {
					nestedCards.push([]);
				}

				// # Split: Cards by row
				for( var column = 0, totalColumns = cardColumns.length; column < totalColumns; column++ ) {
					var columnCards = $( cardColumns[ column ] ).find('.__mst_card');
					for( var card = 0, totalCards = columnCards.length; card < totalCards; card++ ) {
						nestedCards[ card ].push( columnCards[ card ] );
					}
				}

				// # Loop: Each Row
				for( var row = 0, totalRows = nestedCards.length; row < totalRows; row++ ) {

					// # Determine: Minimum Title Heights
					var minHeight = 0;
					var useMinHeight = false;
					for( var card = 0, totalCards = nestedCards[ row ].length; card < totalCards; card++ ) {
						var currentCard = $( nestedCards[ row ][ card ] );
						var titleElem = currentCard.find('.__mst_card_title_text');

						// # Detect: Amount of lines within the text
						var titleElemHeight = titleElem.height();
						var titleElemLineHeight = parseFloat( titleElem.css('line-height') );
						var totalTitleLines = titleElemHeight / titleElemLineHeight;
						
						if( useMinHeight === false && totalTitleLines > 2 ) {
							useMinHeight = true;
						}

						console.log( totalTitleLines );


						if( titleElem.height() > minHeight ) {
							minHeight = titleElem.height();
						}
					}

					// # Apply: Minimum Title Heights using Line-Height
					for( var card = 0, totalCards = nestedCards[ row ].length; card < totalCards; card++ ) {
						var currentCard = $( nestedCards[ row ][ card ] );
						var titleTextElem = currentCard.find('.__mst_card_title_text');
						var titleElem = currentCard.find('.__mst_card_title');

						if( titleTextElem.height() != minHeight ) {
							if( useMinHeight === false ) {
								titleTextElem.css('line-height', minHeight + 'px');
								titleElem.removeClass('__mst_multiline_title');
							} else {
								titleElem.addClass('__mst_multiline_title');
								titleTextElem.css('height', minHeight + 'px');
							}
						}
					}


				}

			},
			equalizeContent: function( selector ) {

				var cardColumns = $( selector ).find('.__mst_cards_column');
				var nestedCards = [];
				var maxCardsPerColumn = 0;
				
				// # Determine: Amount of cards per column to determine rows.
				for( var column = 0, totalColumns = cardColumns.length; column < totalColumns; column++ ) {
					var totalCards = $( cardColumns[ column ] ).find('.__mst_card');
					if( totalCards.length > maxCardsPerColumn ) {
						maxCardsPerColumn = totalCards.length;
					}
				}

				// # Add: Empty arrays to make nested array based on each row
				for( var card = 0, totalCards = maxCardsPerColumn; card < totalCards; card++ ) {
					nestedCards.push([]);
				}

				// # Split: Cards by row
				for( var column = 0, totalColumns = cardColumns.length; column < totalColumns; column++ ) {
					var columnCards = $( cardColumns[ column ] ).find('.__mst_card');
					for( var card = 0, totalCards = columnCards.length; card < totalCards; card++ ) {
						nestedCards[ card ].push( columnCards[ card ] );
					}
				}

				// # Loop: Each Row
				for( var row = 0, totalRows = nestedCards.length; row < totalRows; row++ ) {

					// # Determine: Minimum Title Heights
					var minHeight = 0;
					for( var card = 0, totalCards = nestedCards[ row ].length; card < totalCards; card++ ) {
						var currentCard = $( nestedCards[ row ][ card ] );
						var contentElem = currentCard.find('.__mst_card_content');
						var contentInnerElem = currentCard.find('.__mst_card_content_inner');
						var contentCta = currentCard.find('.__mst_card_cta');

						contentElem.show();
						

						if( contentElem.height() > minHeight ) {
							minHeight = contentElem.height();
						}

					}
					
					// # Apply: Minimum Title Heights using Line-Height
					for( var card = 0, totalCards = nestedCards[ row ].length; card < totalCards; card++ ) {
						var currentCard = $( nestedCards[ row ][ card ] );
						var contentElem = currentCard.find('.__mst_card_content');
						var contentInnerElem = currentCard.find('.__mst_card_content_inner');
						var contentCta = currentCard.find('.__mst_card_cta');

						if( contentCta.length > 0 && contentElem.height() < minHeight ) {
							var newMinHeight = minHeight - contentCta.outerHeight();
							contentInnerElem.css('min-height', newMinHeight + 'px');
						} else if( contentCta.length <= 0 && ( contentElem.height() != minHeight ) ) {
							contentInnerElem.css('min-height', minHeight + 'px');
						}

						
						contentElem.hide();
					}


				}

			},
			// # Function: Make title elements focusable by row order.
			focusableTitles: function( selector ) {

				var cardColumns = $( selector ).find('.__mst_cards_column');
				var nestedCards = [];
				var maxCardsPerColumn = 0;
				var allCards = $( selector ).find('.__mst_card');
				var currentCardCount = 0;
				
				// # Determine: Amount of cards per column to determine rows.
				for( var column = 0, totalColumns = cardColumns.length; column < totalColumns; column++ ) {
					var totalCards = $( cardColumns[ column ] ).find('.__mst_card');
					if( totalCards.length > maxCardsPerColumn ) {
						maxCardsPerColumn = totalCards.length;
					}
				}

				// # Add: Empty arrays to make nested array based on each row
				for( var card = 0, totalCards = maxCardsPerColumn; card < totalCards; card++ ) {
					nestedCards.push([]);
				}

				// # Split: Cards by row
				for( var column = 0, totalColumns = cardColumns.length; column < totalColumns; column++ ) {
					var columnCards = $( cardColumns[ column ] ).find('.__mst_card');
					for( var card = 0, totalCards = columnCards.length; card < totalCards; card++ ) {
						nestedCards[ card ].push( columnCards[ card ] );
					}
				}

				// # Loop: Each Row
				for( var row = 0, totalRows = nestedCards.length; row < totalRows; row++ ) {
					
					for( var card = 0, totalCards = nestedCards[ row ].length; card < totalCards; card++ ) {
						var currentCard = $( nestedCards[ row ][ card ] );
						var cardTitle = currentCard.find('.__mst_card_title_wrapper');
						var cardContent = currentCard.find('.__mst_card_content');
						
						cardTitle.attr('tabindex', '0');
						cardContent.attr('tabindex', '0');
					}
				}

			}
		};

		
		// Run the loop
		pluginMethods.__init( selectedObjects );

		$(window).resize(function() {
			pluginMethods.__reset( selectedObjects );
			pluginMethods.__init( selectedObjects );
		});

	};

}(jQuery));